package Assignments.A4;

public class Quicksort {

    /** Number of comparisons for the sorting alg. */
    public int comps = 0;

    /**
     * Sorts the provided list using quicksort.
     * @param list the list to be sorted.
     */
    public void sort(int[] list) {
        if (list != null && list.length > 0) {
            this.comps = 0;
            quicksort(list, 0, list.length - 1);
        }
    }

    /**
     * This functions sorts the provided array (by reference) using quicksort algorithm.
     * <p>
     * It does this by while high is greater than low, it finds the partitioning index and then
     * calls quicksort again on the lower half and upper half.
     * </p>
     * @param list  the list to be sorted.
     * @param low   the lower bound of the list.
     * @param high  the upper bound of the list.
     */
    private void quicksort(int[] list, int low, int high) {
        if (low < high) {
            this.comps++;
            int pi = partition(list, low, high);
            quicksort(list, low, pi - 1);
            quicksort(list, pi + 1, high);
        }
    }

    /**
     * This function takes the last element as pivot, places the pivot element at its sorted position.
     * @param values the array to be sorted.
     * @param low    the lower bound of the array.
     * @param high   the upper bound of the array.
     * @return the partition index.
     */
    private int partition(int[] values, int low, int high) {
        int pivot = values[high];
        int i = (low - 1);

        for (int j = low; j <= high - 1; j++) {
            if (values[j] < pivot) {
                i++;
                swap(values, i, j);
            }
            this.comps++;
        }
        swap(values, i + 1, high);
        return (i + 1);
    }

    /**
     * Abstracts teh swap function to make the code easier to follow.
     * @param values the array to be sorted.
     * @param i      the first index to be swapped.
     * @param j      the second index to be swapped.
     */
    private void swap(int[] values, int i, int j) {
        int temp = values[i];
        values[i] = values[j];
        values[j] = temp;
    }
}
